-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- Simple Engine Safety System
-- by wlkmanist, 2023

-- notes:
-- electrics.values.throttle
-- engine.thermals.engineBlockTemperature
-- engine.cylinderWallTemperature

local M = {}
M.type = "auxiliary"

local engine = nil
local tempRevLimitState = false
local tempLimitReached = false

local rpmToAV = 0.104719755

-- Variables
local engineName = "mainEngine"
local waterTempLimit = 120                                      -- temperature limit threshold
local oilTempLimit = 130                                        -- temperature limit threshold
local tempHysteresis = 5                                        -- temperature hysteresis
local rpmLimit = 4000                                           -- set temporary this RPM limit to the engine when critical temperature threshold is reached
local throttleLimit = 0.5                                       -- set temporary this throttle limit to the engine when critical temperature threshold is reached
local isEnabled = true                                          -- master switch


local function updateGFX(dt)
  if isEnabled and engine and engine.thermals and engine.thermals.coolantTemperature and engine.thermals.oilTemperature and electrics.values then
    if tempLimitReached then
      tempLimitReached = engine.thermals.coolantTemperature >= waterTempLimit - tempHysteresis or engine.thermals.oilTemperature - tempHysteresis >= oilTempLimit
    else
      tempLimitReached = engine.thermals.coolantTemperature >= waterTempLimit or engine.thermals.oilTemperature >= oilTempLimit
    end
    
    if tempRevLimitState == false and tempLimitReached == true then
      engine:setTempRevLimiter(rpmLimit * rpmToAV)
      tempRevLimitState = true
    elseif tempRevLimitState == true and tempLimitReached == false then
      engine:resetTempRevLimiter()
      tempRevLimitState = false
    end
    
    if tempRevLimitState then
      electrics.values.throttle = math.min(throttleLimit, electrics.values.throttle)
      electrics.values.checkengine = true
    end
  end
end

local function reset()
  if engine then
    engine:resetTempRevLimiter()
    tempRevLimitState = false
  end
end

local function init(jbeamData)
  if jbeamData.engineName ~= nil then
    engineName = jbeamData.engineName
  end
  if jbeamData.rpmLimit ~= nil then
    rpmLimit = jbeamData.rpmLimit
  end
  if jbeamData.waterTempLimit ~= nil then
    waterTempLimit = jbeamData.waterTempLimit
  end
  if jbeamData.oilTempLimit ~= nil then
    oilTempLimit = jbeamData.oilTempLimit
  end
  if jbeamData.tempHysteresis ~= nil then
    tempHysteresis = jbeamData.tempHysteresis
  end
  if jbeamData.throttleLimit ~= nil then
    throttleLimit = jbeamData.throttleLimit
  end
  if jbeamData.isEnabled ~= nil then
    isEnabled = jbeamData.isEnabled
  end
  
  engine = powertrain.getDevice(engineName)
  M.updateGFX = engine and updateGFX or nop
  tempRevLimitState = false
end


M.init = init
M.reset = reset
M.updateGFX = nop

return M
