-- Some other libraries or additions used in this project can have
-- theirs own licences.
--
-- Copyright (C) 2024 wlkmanist <vlad.king5555@mail.ru> - All Rights Reserved.
--
-- This program is free software; you can redistribute it and/or modify it
-- under the terms and conditions of the GNU General Public License, version 3,
-- as published by the Free Software Foundation.
--
-- Contact me directly if you want to use the code below in your project some way.
-- Feel free to use this code without any restrictions if you are BeamNG developer.


local M = {}
local torqueDataModifiers = {}
local modifiersCounter = 0
local engine = nil

--local isExisting = false

--local function init(jbeamData)
--  log("D", "advanced_internals.torqueCurveManager", "Init")
--
--  local engineName = nil
--  if jbeamData.engineName ~= nil then
--    engineName = jbeamData.engineName
--  else
--    engineName = "mainEngine"
--  end
--
--  if powertrain then
--    engine = powertrain.getDevice(engineName)
--  end
--
--  engine.sendTorqueData = M.sendTorqueData                        -- use ours function
--end

--local function onReset(device)
--  log("D", "advanced_internals.torqueCurveManager", "onReset()")
--  M.sendTorqueData(engine)                                        -- force apply curves
--end

local function sendTorqueData(device, data)
  log("D", "advanced_internals.torqueCurveManager", "sendTorqueData()")
  if not data then
    data = device:getTorqueData()                                   -- vanilla implementation
    
    if torqueDataModifiers then                                     -- torqueData pushes through all custom modifiers functions and can be fully edited in each one
      for i = 1, modifiersCounter do                                -- all added custom "curves"
        data = torqueDataModifiers[i](data)
      end
    end
  end

  guihooks.trigger("TorqueCurveChanged", data)
end

local function addTorqueDataModifier(func)                          -- adds custom modifier funtion from public
  log("D", "advanced_internals.torqueCurveManager", "Adding new torque data modifier...")
  modifiersCounter = modifiersCounter + 1
  table.insert(torqueDataModifiers, modifiersCounter, func)
end

local function getDevice()
  return engine
end

local function setDevice(device)                                    -- torqueCurveManager is enabled after setting device
  if device ~= nil then
    engine = device
    engine.sendTorqueData = M.sendTorqueData                        -- use ours function instead of vanilla one
    log("D", "advanced_internals.torqueCurveManager", "Started")
  end
end


-- public interface
M.sendTorqueData = sendTorqueData
M.addTorqueDataModifier = addTorqueDataModifier
M.getDevice = getDevice
M.setDevice = setDevice
--M.init = init
--M.onReset = onReset

return M
